"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyGenerators = exports.createCluster = exports.assignPersons = exports.upsert = exports.createGroupActivity = void 0;
const helpers_1 = require("@wca/helpers");
const createGroupActivity = (id, roundActivity, groupNumber, startTime, endTime) => {
    const { eventId, roundNumber, attemptNumber } = (0, helpers_1.parseActivityCode)(roundActivity.activityCode);
    const activityCode = `${eventId}-r${roundNumber}-g${groupNumber}` +
        (attemptNumber ? `-a${attemptNumber}` : '');
    return {
        id,
        name: (0, helpers_1.activityCodeToName)(activityCode),
        activityCode,
        startTime: startTime || roundActivity.startTime,
        endTime: endTime || roundActivity.endTime,
        childActivities: [],
        extensions: [],
    };
};
exports.createGroupActivity = createGroupActivity;
/**
 * Creates a new version of data with the data from initial for each item where the `id` prop matches.
 */
const upsert = (initial, data) => {
    return initial.map((item) => {
        const match = data.find((d) => d.id === item.id);
        return match ? match : item;
    });
};
exports.upsert = upsert;
/**
 * Returns a new person array with person's assignments generated based on the given constraints
 */
const assignPersons = ({ wcif, cluster, activities, constraints, assignmentCode, options, }) => {
    return cluster.reduce((persons, person) => {
        const activityScores = new Map();
        activities.forEach((activity) => {
            activityScores.set(activity.id, 0);
            const scores = constraints.map(({ constraint, weight, options: constraintOptions }) => {
                const score = constraint.score({
                    wcif: Object.assign(Object.assign({}, wcif), { persons }),
                    activities,
                    cluster,
                    assignmentCode,
                    person,
                    activity,
                    options: Object.assign(Object.assign({}, options), constraintOptions),
                });
                return score === null ? score : score * weight;
            }) || [0];
            if (scores.some((score) => score === null)) {
                activityScores.delete(activity.id);
            }
            else {
                const score = scores.reduce((a, b) => a + b, 0);
                activityScores.set(activity.id, score);
            }
        });
        if (activityScores.size === 0) {
            // No changes being made, return the original person.
            return persons;
        }
        // find the activity with the highest score
        const bestActivityId = [...activityScores.entries()].reduce((a, b) => a[1] > b[1] ? a : b)[0];
        return persons.map((p) => p.registrantId === person.registrantId
            ? Object.assign(Object.assign({}, person), { assignments: [
                    ...(person.assignments || []),
                    {
                        activityId: bestActivityId,
                        stationNumber: null,
                        assignmentCode,
                    },
                ] }) : p);
    }, wcif.persons);
};
exports.assignPersons = assignPersons;
/**
 * A cluster is a subset of persons.
 * The default constraints require that the registraiton status is accepted
 *
 * @deprecated
 * @param persons
 * @param options
 * @returns
 */
const createCluster = (persons, options = {}) => {
    return persons.filter((p) => {
        var _a, _b;
        if (((_a = p.registration) === null || _a === void 0 ? void 0 : _a.status) !== 'accepted') {
            return false;
        }
        // Filter for first timers
        if (options.firstTimer === true && p.wcaId) {
            return false;
        }
        else if (options.firstTimer === false && !p.wcaId) {
            return false;
        }
        // Filter for roles
        if (((_b = options.roles) === null || _b === void 0 ? void 0 : _b.length) &&
            // if the person has none of the roles
            !options.roles.some((r) => { var _a; return (_a = p.roles) === null || _a === void 0 ? void 0 : _a.includes(r); })) {
            return false;
        }
        return true;
    });
};
exports.createCluster = createCluster;
const applyGenerators = ({ wcif: initialWcif, generators, }) => generators.reduce((wcif, { generator, props }) => generator.execute(Object.assign({ wcif }, props)), initialWcif);
exports.applyGenerators = applyGenerators;
//# sourceMappingURL=utils.js.map